@@ -197,7 +197,7 @@ def event_listener(module, event, count):
 
 def test_naverkin(keywords, page_range):
     nkc = NaverKinCrawler(keywords, page_range,
-        settings=settings
+        settings=settings,crawl_listener=event_listener
     )
-    nkc.crawl(crawl_listener=event_listener)
+    nkc.crawl()
     return nkc
